/*
 * PLUGIN DE ROTATION POUR INTERNET EXPLORER SEULEMENT
 * Laurent-Yann Saint-Dizier - mars 2012 - version 1.3
 *
 * Licensed under the MIT license:
 * http://www.opensource.org/licenses/mit-license.php
 * 
 * Le filtre de rotation a un premier bug car le centre de la rotation
 * n'est pas au centre de l'objet. Il faut faire une translation
 * or le filtre a un second bug car il ne fait pas cette translation !
 * Il faut donc faire la rotation via le filtre puis calculer la translation
 * et la faire en CSS (via jQuery)
 */
if( typeof jQuery!="undefined") {
	if(!jQuery.browser.msie) jQuery.error("Ce plugin de rotation ne peut être utilisé qu'avec Internet Explorer !")
	if( typeof jQuery.fn.translate=="function") jQuery.error("Le plugin de translation est déjà défini !");
	if( typeof jQuery.fn.rotate=="function") jQuery.error("Le plugin de rotation est déjà défini !");
	// on surcharge la fonction d'opacité de jQuery pour enregistrer la valeur finale de l'opacité
	// ça permettra de corriger le bug d'opacité via style.filter
	var old_step_opacity=jQuery.fx.step.opacity;
	jQuery.fx.step.opacity= function(a) {
		jQuery(a.elem).attr("jQueryOpacity", a.end); // on créé un nouvel attribut contenant la valeur de l'opacité finale
		return old_step_opacity(a)
	};
}

(function($) {
	/*
	 * Permet de déplacer un objet suivant un vecteur {left:x,top:y}
	 */	
	$.fn.translate = function(offset_top_left_object) {
		return this.each(function() {
	        var elem = this;
	        var $elem = $(elem);
	        $elem.wrap('<div style="position: relative" />');
	        var pos = $elem.position();
	    	$elem.unwrap();
	    	$elem.css({
		        top: parseInt(pos.top*1 + offset_top_left_object.top*1)+"px",
		        left: parseInt(pos.left*1 + offset_top_left_object.left*1)+"px"
		    });
	    });
	};
	
	$._tab_cos = []; $._tab_sin = []; // utilisés pour stocker les résultats de cos() et sin()
	
	/*
	 * Permet de tourner un objet de x degrés et d'appeler une fonction quand c'est fait
	 */
	$.fn.rotate= function(angle_en_degres,callback) {
		var _delay_timeout=5;
		// on redéfinie les fonctions sin() et cos() pour qu'elles prennent des degrés au lieu des radians
		var _SIN = function(angle_en_degres) {
			var angle = angle_en_degres % 360;
			if (typeof($._tab_sin[angle])=="undefined") {
				// le sinus n'a pas encore été calculé, on va ensuite le mémoriser
				$._tab_sin[angle] = Math.sin(Math.PI*angle/180);
			}
			return $._tab_sin[angle];
		};
		var _COS = function(angle_en_degres) {
			var angle = angle_en_degres % 360;
			if (typeof($._tab_cos[angle])=="undefined") {
				// le cosinus n'a pas encore été calculé, on va ensuite le mémoriser
				$._tab_cos[angle] = Math.cos(Math.PI*angle/180);
			}
			return $._tab_cos[angle];
		};
		// permet d'avoir des nombres à virgule "sains" (pas de 0.000000000000001)
		var fixFloat= function(a) {
			return 1.0*parseFloat(parseFloat(a).toFixed(8))
		};
		var fixInt= function(a) {
			return 1*parseInt(""+Math.ceil(fixFloat(a)))
		};
		/*
		 * calcule les coordonnées du centre d'un segment
		 */
		var milieu = function(point1,point2) {
			return {left:fixInt(point1.left/2+point2.left/2), top:fixInt(point1.top/2+point2.top/2)};
		};
		/*
		 * calcule les coordonnées d'un vecteur
		 */
		var vecteur = function(point1,point2) {
			return {left:fixInt(point2.left-point1.left), top:fixInt(point2.top-point1.top)};
		};
		/*
		 * permet de supprimer les mentions d'opacité et de rotation dans une chaine de filtre
		 */
		var cleanFilter= function(a) {
			var b=""+a, c=/alpha[(]opacity=[^)]+[)][;]?/i;
			if(b!=""&&c.test(b)) {
				// on supprime les mentions d'opacité
				var d=c.exec(b);
				for(var e=0 ; e<d.length ; e++) {
					b=b.replace(d[e]+";", " ");
					b=b.replace(d[e], " ")
				}
				b=jQuery.trim(b)
			}
			c=/progid:DXImageTransform[.]Microsoft[.]Matrix[(][^)]+[)]/i;
			if(b!=""&&c.test(b)) {
				// on supprime les mentions de rotation
				var d=c.exec(b);
				for(var e=0 ; e<d.length ; e++) {
					b=b.replace(d[e], " ")
				}
				b=jQuery.trim(b)
			}
			return b
		};
		/*
		 * retourne la chaine de filtre correspondant à la matrice de rotation
		 * (la translation Dx,Dy ne fonctionne pas mais on la met quand même !)
		 */
		var rotation_matrix = function(matrix,dx,dy) {
			// matrix : [[a,b],[c,d]];
			return  " progid:DXImageTransform.Microsoft.Matrix(M11="+ matrix[0][0] +", M12="+ matrix[1][0] +
					", M21="+ matrix[0][1] +", M22="+ matrix[1][1] +", Dx="+ dx +", Dy="+ dy +", sizingMethod='auto expand') ";
		};
		/*
		 * fait la rotation
		 */
		var ie_rotate= function(elem, angle, callback) {
			var $elem=$(elem);
			if( typeof $elem.data("rotate_coord")=="undefined") {
				// on mémorise les infos originales sur l'objet
				var opa=""+$(elem).attr("jQueryOpacity"), 
				f=cleanFilter($elem[0].style.filter);
				if(isNaN(opa)) opa=1; else opa=parseFloat(opa);
				var pos=$elem.position(), 
					w=$elem.outerWidth(), 
					h=$elem.outerHeight(), 
					mid=milieu(pos, {left: pos.left+w, top: pos.top+h}), 
					coord="{left:"+pos.left+",top:"+pos.top+",width:"+w+",height:"+h+
					",width2:"+fixFloat(w/2)+",height2:"+fixFloat(h/2)+",mid:{left:"+mid.left+",top:"+mid.top+"}"+
					",opacity:"+opa+",filter:"+'"'+f+'"'+"}";
				$elem.data("rotate_coord", coord);
				$elem.data("rotate_mid",mid.left+"/"+mid.top); // on mémorise le centre initial de l'objet
				eval("coord="+coord+";")
			} else {
				// on retrouve les infos originales mémorisées sur l'objet
				var coord=$elem.data("rotate_coord");
				eval("coord="+coord+";")
			}
			// on calcule la matrice de rotation
			var costheta=_COS(angle), 
				sintheta=_SIN(angle), 
				a=fixFloat(costheta), b=fixFloat(sintheta), 
				c=fixFloat(-sintheta), d=fixFloat(costheta), 
				tx=fixInt(-coord.width2*costheta+coord.height2*sintheta+coord.width2), 
				ty=fixInt(-coord.width2*sintheta-coord.height2*costheta+coord.height2), 
				matrix=[[a, b], [c, d]];
				
			// et suivant l'angle on fait différents traitements...
			
			if(angle!=0 && angle!=180) {
				// on fait la rotation normalement
				$elem[0].style.filter=rotation_matrix(matrix, tx, ty);
				// on replace l'objet à l'emplacement d'origine et aux bonnes dimensions
				$elem.css({
					left: coord.left+"px", top: coord.top+"px",
					width: coord.width+"px", height: coord.height+"px"
				});
			}
			
			
			if(angle==0) {
				
				$elem.css({
					left: coord.left+"px", top: coord.top+"px",
					width: coord.width+"px", height: coord.height+"px"
				});
				$elem[0].style.filter=coord.filter;
				if(0<coord.opacity&&coord.opacity<1) $elem.css({opacity: coord.opacity});
				//$elem.data("rotate_mid",mid.left+"/"+mid.top); // on met à jour le centre de l'objet tourné
				if(typeof(callback)=="function") callback()
				
			} else if(0<angle&&angle<90) {
				
				// on utilise un 'setTimeout' pour avoir le temps que $elem.height() renvoie la bonne mesure
				setTimeout(function() {
					// l'objet initial est un rectangle A1B1C1D1, l'objet tourné est aussi un rectangle A2B2C2D2
					// on va chercher les coordonnées des centres X1 et X2 de ces rectangles
					// on fera alors la translation du vecteur X2X1 pour que la rotation retrouve le "bon centre"
					var x1=coord.mid, 
						x2=milieu({
							left: coord.left + coord.width*costheta,
							top: coord.top + $elem.height()
						}, {
							left: coord.left + coord.height*sintheta,
							top: coord.top
						}),
						translation=vecteur(x2, x1);
					// on fait la rotation
					$elem[0].style.filter=coord.filter+rotation_matrix(matrix, tx, ty);
					// on corrige le bug du filtre pour le centre de la rotation
					$elem.translate(translation);
					// on corrige le bug de l'opacité
					if(0<coord.opacity&&coord.opacity<1) $elem.css({opacity: coord.opacity});
					//$elem.data("rotate_mid",x2.left+"/"+x2.top); // on met à jour le centre de l'objet tourné
					// la rotation est terminée, on peut lancer la fonction de callback si elle existe
					if(typeof(callback)=="function") callback();
				}, _delay_timeout)
				
			} else if(angle==90||angle==270) {
				
				setTimeout(function() {
					var x1=coord.mid, 
						x2=milieu({
							left: coord.left + coord.height,
							top: coord.top
						}, {
							left: coord.left,
							top: coord.top + coord.width
						}),
						translation=vecteur(x2, x1);
					$elem[0].style.filter=coord.filter+rotation_matrix(matrix, tx, ty);
					$elem.translate(translation);
					if(0<coord.opacity&&coord.opacity<1) $elem.css({opacity: coord.opacity});
					if(typeof(callback)=="function") callback()
				}, _delay_timeout)
				
			} else if(90<angle&&angle<180) {
				
				setTimeout(function() {
					costheta=_COS(angle-90); sintheta=_SIN(angle-90);
					var x1=coord.mid, 
						x2=milieu({
							left: coord.left + sintheta*coord.width,
							top: coord.top
						}, {
							left: coord.left + costheta*coord.height,
							top: coord.top + $elem.height()
						}),
						translation=vecteur(x2, x1);
					$elem[0].style.filter=coord.filter+rotation_matrix(matrix, tx, ty);
					$elem.translate(translation);
					if(0<coord.opacity&&coord.opacity<1) $elem.css({opacity: coord.opacity});
					if(typeof(callback)=="function") callback()
				}, _delay_timeout)

			} else if(angle==180) {
				
				$elem.css({
					left: coord.left+"px", top: coord.top+"px",
					width: coord.width+"px", height: coord.height+"px"
				});
				$elem[0].style.filter=coord.filter+rotation_matrix(matrix, tx, ty);
				if(0<coord.opacity&&coord.opacity<1) $elem.css({opacity: coord.opacity});
				if(typeof(callback)=="function") callback()				

			} else if(180<angle&&angle<270) {
				
				setTimeout(function() {
					costheta=_COS(angle-180); sintheta=_SIN(angle-180);
					var x1=coord.mid, 
						x2=milieu({
							left: coord.left + $elem.width(),
							top: coord.top + coord.width*sintheta
						}, {
							left: coord.left,
							top: coord.top + coord.height*costheta
						}),
						translation=vecteur(x2, x1);
					$elem[0].style.filter=coord.filter+rotation_matrix(matrix, tx, ty);
					$elem.translate(translation);
					if(0<coord.opacity&&coord.opacity<1) $elem.css({opacity: coord.opacity});
					if(typeof(callback)=="function") callback()				
				}, _delay_timeout)

			} else if(270<angle&&angle<360) {
				
				setTimeout(function() {
					costheta=_COS(angle-270); sintheta=_SIN(angle-270);
					var x1=coord.mid, 
						x2=milieu({
							left: coord.left + costheta*coord.height,
							top: coord.top + $elem.height()
						}, {
							left: coord.left + sintheta*coord.width,
							top: coord.top
						}),
						translation=vecteur(x2, x1);
					$elem[0].style.filter=coord.filter+rotation_matrix(matrix, tx, ty);
					$elem.translate(translation);
					if(0<coord.opacity&&coord.opacity<1) $elem.css({opacity: coord.opacity});
					if(typeof(callback)=="function") callback()
				}, _delay_timeout)

			}
		};
		
		// on ne met pas de 'return' car il y a un setTimeout (pour avoir le temps de récupérer les valeurs des filtres)
		this.each(function() {
			var elem=this;
			setTimeout(function() {
				ie_rotate(elem, angle_en_degres%360, callback)
			}, _delay_timeout)
		});
		
	};
	
	
})(jQuery);